#ifndef _BulletFieldMgr_
#define _BulletFieldMgr_


int		BulletFieldCreate(DialogPtr Dialog, int NumItems, int FieldCount, int Field1, ...);
void	BulletFieldDispose(DialogPtr Dialog);




/*	
	Bullet Field Manager -- Procedure Summary
	
	
		BulletFieldCreate
		BulletFieldDispose
	
	
	
	BulletFieldCreate
	
	
	int BulletFieldCreate(DialogPtr Dialog, int NumItems, int FieldCount, 
		int Field1, ...);
	
	BulletFieldCreate sets-up one or more bulleted fields in the dialog 
	specified by Dialog.  The fields to be bulleted are specified by a 
	variable number of arguments beginning with Field1 - it and all 
	following parameters should be the numbers of items in the dialog
	item list that are to be bulleted.  FieldCount should be the
	number of items to be bulleted, i.e. the total number of parameters 
	beginning with Field1 and including all following parameters.  The 
	value in NumItems is added to each of the item numbers specified 
	in Field1 (and all following) parameters.  This is handy when 
	writing cdevs, but will be of no use in conventional dialogs; just 
	set it to zero in cases where you don't need it.
	
	In order to setup a single bulleted field in item 7 of a dialog 
	you've created which is pointed to by a variable named MyDialog, 
	you'd invoke BulletFieldCreate in the following manner:
	
	BulletFieldCreate(MyDialog, 0, 1, 7);
	
	If, instead, you wanted to setup two bulleted fields in items 7 and 
	13 in that same dialog, you'd make the call this way:
	
	BulletFieldCreate(MyDialog, 0, 2, 7, 13);
	
	Note that FieldCount (the third parameter) is now 2, because two
	dialog item numbers follow it in the parameter list.
	
	BulletFieldCreate returns TRUE if it successfully sets-up the 
	requested bulleted fields.  If it fails for some reason, it will
	return FALSE.
	
	WARNING:  The Bullet Field Manager uses the refCon field of the
			  dialog window to store a pointer to its internal data.
			  If you use the refCon field for other purposes, you'll
			  need to modify all references (there's only 3 of them)
			  to the refCon field in the "Bullet Field Mgr.c" file 
			  in order to allow for your usage.
	
	
	
	BulletFieldDispose
	
	
	void BulletFieldDispose(DialogPtr Dialog);
	
	BulletFieldDispose reverses the effects of BulletFieldCreate, and
	deallocates all the storage it allocated.  Call BulletFieldDispose
	just before disposing of the dialog itself.
	
	
	
		*/


#endif