/*
 * rsrc.h - #defines, etc, which need to stay synched with the resource
 * source file (RevRdist.r)
 */

#define	RSRC_BASE	12100		/* base for resource numbering */

#define CREATOR		'Dist'		/* our creator id */
#define	TYPE_CONT	'TEXT'		/* file type of control file */
#define TYPE_PREF	'RDsp'		/* file type of prefs file */
#define	TYPE_TEMP	'Junk'		/* file type of temp files */
#define	TYPE_LONG	'Long'		/* rsrc type of misc params */
#define	TYPE_TEXT	'TEXT'		/* rsrc type of extended text */

#define	RSRC_MOUNT	2000		/* resid for mount XCMD */
#define	RSRC_UNMOUNT (RSRC_MOUNT+1)	/* and for unmount XCMD */
#define	RSRC_INITICON	131		/* resid of INIT's ICN# */
#define	RSRC_ABOUT	(RSRC_BASE)	/* resid of About text */

/*
 * Misc parameter resource ids
 */

enum
{
	 TIME_INTERVAL = RSRC_BASE	/* seconds between runs */
	,JUNK_PARM					/* min/max junk residence times & space */
	,FLAG_PARM					/* initial value for Flags */
};

/*
 * Window refCons, add RSRC_BASE to get WIND resource id
 */

enum
{
	 WIND_STATUS = 1			/* pause/continue/cancel dialog */
	,WIND_ACTIVITY				/* window for listing of actions as they
								 * occur */
	,WIND_ERRORS				/* error message window */
	,WIND_PREF					/* preferences dialog */
	,ALERT_WARN					/* warning which can be ignored */
	,ALERT_PANIC				/* warning which cannot be ignored */
	,WIND_FLAGS					/* dialog for setting flags */
	,WIND_ABOUT					/* about dialog */
	,ALERT_SAVE					/* save file before closing */
};

/*
 * Item numbers
 */

enum
{								/* in File menu */
	 FILE_OPEN = 1				/* Open ... */
	,FILE_SAVE					/* Save preferences */
	,FILE_SAVEAS				/* Save preferences as ... */
	,FILE_QUIT					/* Quit */
	,FILE_CLOSE					/* dummy */
};

enum
{								/* in Edit menu */
	 EDT_UNDO = 1
	,EDT_CUT = 3
	,EDT_COPY
	,EDT_PASTE
	,EDT_CLEAR
};

enum							/* in status dialog */
{
	 STS_PAUSE = 1				/* pause button */
	,STS_CANCEL					/* cancel button */
	,STS_CONT					/* continue button */
	,STS_STATUS					/* current status text */
	,STS_WAIT					/* please wait text */
	,STS_REBOOT					/* reboot text */
	,STS_PICT					/* our picture */
};

enum							/* in preferences dialog */
{
	 PRF_GO = 1					/* go to it button */
	,PRF_CANCEL					/* cancel button */
	,PRF_ZONE = 4				/* AppleShare zone name */
	,PRF_SRVR = 6				/* AppleShare server name */
	,PRF_USER = 8				/* AppleShare user login name */
	,PRF_PASS = 10				/* User's password */
	,PRF_DISTF = 12				/* distribution control file editText */
	,PRF_MASTF = 14				/* master folder on server editText */
	,PRF_JUNKF = 16				/* junk folder name editText */
	,PRF_INTERVAL = 18			/* update interval editText */
	,PRF_JMIN = 20				/* minimum junk lifetime editText */
	,PRF_JMAX = 22				/* maximum junk lifetime editText */
	,PRF_DMIN = 24				/* free space threshold editText */
	,PRF_LAST = 24				/* highest editText item */
};

/*
 * alert items
 */
enum
{
	 ALRT_OKAY = 1				/* okay button */
	,ALRT_CANCEL				/* cancel button */
	,ALRT_TEXT					/* alert text */
};

enum
{
	 SAVE_YES = 1				/* yes (save) button */
	,SAVE_NO					/* no (don't save) button */
	,SAVE_CANCEL				/* cancel close or quit */
	,SAVE_TEXT					/* alert text */
};


/*
 * Messages in STR# RSRC_BASE+0 - most error messages
 */
#define	ERR_STR		(RSRC_BASE)
enum error_e
{
	 E_SYS = 1	/* When ^0 called ^1, the system returned error ^2 */
	,E_FILE		/* ^0 had problems with ^1.  Error ^2 */
	,E_QUIT		/* Quitting per user request */
	,E_NOMEM	/* Out of memory in ^0 (hint: ^1) */
	,E_REQUEST	/* Internal error: bad dispatcher request/result ^1 in ^0 */
	,E_STATE	/* Internal error: bad dispatcher state in ^0: ^1 */
	,E_DISTF	/* Cannot locate distribution control file: ^0 */
	,E_DISTL	/* Syntax error(s) in directives file */
	,E_PREFIX	/* Bad prefix character in line  */
	,E_NOFNAME	/* Missing file/folder name in line  */
	,E_BADNAME	/* Illegal file/folder name in line  */
	,E_NOACTION	/* Missing action list in line  */
	,E_BADACTION/* Unrecognized action option in line  */
	,E_BADTYPE	/* Bad file TYPE(CREATOR) specification */
	,E_ALTNAME	/* Garbage at end of line  */
	,E_DUPENAME	/* File/folder specified more than once  */
	,E_ENDFERR	/* Internal error: corrupt dist_node at end of folder mark ^0: ^1 */
	,E_DISTBAD	/* Folder mismatch found by ^0 at\0D^1 */
	,E_FIRST	/* First entry in directives file must be root folder */
	,E_LAST		/* Directives file has unmatched folder entries */
	,E_PREFS	/* Preferences file (^0) missing or corrupt */
	,E_NOPREF	/* No preferences file found - should defaults be used? */
	,E_NOVOL	/* Cannot determine which volume to update */
	,E_NOPRINT	/* Printing of control files not supported - open instead? */
	,E_HFS		/* Sorry, ^0 requires the Hierarchical File System */
	,E_MFS		/* Sorry, ^0 requires HFS volumes.  ^1 is MFS. */
	,E_LISTONLY	/* Updates disabled.  Continue, but only list actions? */
	,E_NOSERVER	/* Cannot find master folder (^0) */
	,E_NOACCESS	/* Must have "See files" and "See folders" access to master folder (^0) */
	,E_NOJUNK	/* Cannot open "junk" folder (^0) */
	,E_ACCESS	/* No access to folder (^0) */
	,E_NAMELEN	/* Internal error: name too long in ^0 (^1) */
	,E_BADJUNKF	/* Bad or missing "junk" folder STR resource */
	,E_BADTEMPF	/* Bad or missing temp file STR resource */
	,E_TYPE		/* Cannot save preferences to ^0: wrong file type */
	,E_DOQUIT	/* Quitting now may damage files.  Okay to ignore quit and continue? */
	,E_REMOUNT	/* Could not remount volumes due to system error ^0 */
	,E_MISSING	/* Missing from master: ^0 */
	,E_MISTYPE	/* Control file does not match master for ^0 */

};

/*
 * Messages for the activity listing window
 */
#define	LST_STR		(RSRC_BASE+1)
enum activity_e
{
	 L_SYS = 1	/* When ^0 called ^1, the system returned error ^2 */
	,L_FILE		/* ^0 had problems with ^1.  Error ^2 */
	,L_SKIP		/* Skipping emptying of junk folder (^0) */
	,L_DOJUNK	/* Emptying junk folder (^0) */
	,L_STARTF	/* Working on ^0 */
	,L_ENDF		/* Done with ^0 */
	,L_WCREATE	/* Would create folder ^0 */
	,L_IGNORE	/* Ignoring ^0 */
	,L_CRITERION/* Used ^0^1 for ^2 */
	,L_MISSING	/* Missing from master: ^0 */
	,L_SKPJUNK	/* Skipping junking of ^0 */
	,L_JUNKING	/* Moving ^0 to junk folder */
	,L_COPYING	/* Copying ^0 byte file ^1 to ^2 */
	,L_DOEMPTY	/* Discarding contents of folder ^0 */
	,L_DISCARD	/* Discarding ^0 */
	,L_NOWHAT	/* Don't know what to do with ^0 */
	,L_ACCESS	/* No access to folder (^0) */
	,L_FINFO	/* Updating Finder info for ^0 */
	,L_DTCOMM	/* Copying comment for ^0 */
	,L_DTICON	/* Copying icons for ^0 */
	,L_DTINFO	/* Updated Desktop info for ^0 */
	,L_DTDEL	/* Deleted Desktop info for ^0 */
	,L_TOOMANYJ	/* Too many copies of "^0" in junk folder */
	,L_MISTYPE	/* Control file does not match master for ^0 */
	,L_ALIAS	/* Updating alias to ^0 */
	,L_BLESSED	/* Setting Blessed folder */
	,L_BADBLESSED	/* Volume ^0 missing ^1.  It might not boot. */
	,L_PROTECT	/* Cannot preserve protection on ^0.  File hidden. */
	,L_DONE		/* RevRdist complete */
};

/*
 * STR  resource id's
 */
enum {
	 STR_PREFS = RSRC_BASE		/* name of preferences file */
	,STR_DISTF					/* name of distribution control file */
	,STR_ZONE					/* name of server zone */
	,STR_SRVR					/* name of server node */
	,STR_USER					/* user name on server */
	,STR_PASS					/* password of user on server */
	,STR_MASTF					/* name of server master folder */
	,STR_JUNKF					/* name of junk folder */
	,STR_JUNKSUF				/* suffix to uniquify junk files */
	,STR_UNTITLED				/* name for document with no file */
	,STR_PSAVE					/* text for prefs SFPutFile */
	,STR_START					/* name of startup application */
	,STR_TEMP1					/* first temp file name */
	,STR_TEMP2					/* second temp file name */
};

/*
 * Numbers for NAME_STR strings
 */
#define	NAME_STR	(RSRC_BASE+2)
enum {
	 NAME_FOLDER = 1			/* name of system folder */
	,NAME_SYSTEM				/* name of system file */
	,NAME_FINDER				/* name of shell file */
};

/*
 * Numbers for MISC_STR strings
 */
#define	MISC_STR	(RSRC_BASE+3)
enum {
	 ALIAS_WORD = 1				/* distfile word for alias */
	,ALIAS_INFO					/* pseudo-type/creator for Finder aliases */
	,ICON_WORD					/* distfile word for icon files */
	,ICON_INFO					/* pseudo-type/creator for Finder icon files */
	,ICON_FNAME					/* name Finder uses for icon files */
};